#include<stdio.h>
#include"card.h"
#include"console.h"
#include"gawk.h"

//Generate an ordered deck of 52 cards (no jokers)
struct card *generate_deck(void)
{
	struct card* deck;
	struct card* tmp;
	enum suit suit;
	enum rank rank;
	deck = (struct card *) malloc(sizeof(struct card)*52);
	tmp = deck;
	for(rank=ACE;rank<=KING;rank++){
		for(suit=HEART;suit<=SPADE;suit++){
		//	printf("%d %c\n", rank, suit);
			tmp->s = suit;
			tmp->r = rank;
			tmp->t = FACE_DOWN;
			tmp->prev = tmp-1;
			tmp->next = ++tmp;
		}
	}
	(--tmp)->next=NULL;
	deck->prev=NULL;
	return(deck);
}

void card_to_string(struct card* card, char *string)
{
	if(card==NULL){
		strcpy(string, "   ");
		return;
		//printf("Error card is NULL\n");
		//exit(1);
	}
	switch(card->r){
		case ACE:
			string[0]=' ';
			string[1]='A';
			break;
		case TWO:
		case THREE:
		case FOUR:
		case FIVE:
		case SIX:
		case SEVEN:
		case EIGHT:
		case NINE:
			string[0]=' ';
			string[1]='0'+card->r;
			break;
		case TEN:
			string[0]='1';
			string[1]='0';
			break;
		case JACK:
			string[0]=' ';
			string[1]='J';
			break;
		case QUEEN:
			string[0]=' ';
			string[1]='Q';
			break;
		case KING:
			string[0]=' ';
			string[1]='K';
			break;
		default:
			printf("Error card is invalid rank: %d\n", card->r);
			exit(1);
	}
	switch(card->s){
		case HEART:
		case DIAMOND:
		case CLUB:
		case SPADE:
			string[2]=card->s;
			break;
		default:
			printf("Error card is invalid suit: %d\n", card->s);
	}
	string[3]=0;
	if(card->t==FACE_UP){
		return;
	}
	else{
		strcpy(string,"xxx");
		return;
	}
}

//Print out one card to stdout
void print_card(struct card* card)
{
	char string[4];
	card_to_string(card, string);
//	printf("%s", string);
	print_text(string, get_color(card));
}

//Print a deck of cards to stdout
void print_deck(struct card* deck)
{
	struct card* tmp;
	int i;
	tmp = deck;
	i=0;
	while(tmp!=NULL){
		print_card(tmp);
		printf(" ");
		i++;
		tmp=tmp->next;
		if(i==12){
			printf("\n");
			i=0;
		}
	}
	printf("\n");


}

//Prints a deck of cards to stdout starting from the end
void print_deck_backwards(struct card* deck)
{
	struct card* tmp;
	int i;
	tmp=deck;
	while(tmp->next!=NULL){
		tmp=tmp->next;
	}
	i=0;
	while(tmp!=NULL){
		print_card(tmp);
		printf(" ");
		i++;
		tmp=tmp->prev;
		if(i==12){
			printf("\n");
			i=0;
		}

	}

	printf("\n");

}

//Returns length of deck (number of cards in deck)
int get_length_of_deck(struct card* deck)
{
	int i=0;
	while(deck!=NULL){
		i++;
		deck=deck->next;
	}
	return(i);
}

//Returns 0 if card is red (heart, diamond) and 1 if black (club, spade)
//Returns -1 if card is NULL or card is face down
int get_color(struct card* card)
{
	if(card==NULL || card->t==FACE_DOWN){
		return(-1);
	}
	if(card->s==HEART || card->s==DIAMOND){
		return(0);
	}
	else if(card->s==CLUB || card->s==SPADE){
		return(1);
	}
	else{
		printf("Card has invalid suit: %d\n", card->s);
		exit(1);
	}
}

//Generate a new shuffled deck from an existing deck
struct card *shuffle_deck(struct card* original)
{
	struct card* deck;
	struct card* tmp;
//	char string[10];

	int length;

	int size=52;
	int a[52];
	int i;
	int r;

	length = get_length_of_deck(original);
	if(length!=52){
		printf("Error original deck has wrong number of cards: %d\n",
				length);
		exit(1);
	}
//	system("gawk -f random.awk>rand.txt");
//	in=fopen("rand.txt", "r");
//	if(!in){
//		printf("Error failed to open rand.txt\n");
//		exit(1);
//	}
	deck = (struct card *) malloc(sizeof(struct card)*52);
	tmp = original;
//	while(fgets(string,10,in)){
//		rand=atoi(string);
//		if(rand<0 || rand>51){
//			printf("Error random number out of range: %d\n", rand);
//			exit(1);
//		}
	for(i=0;i<size;i++){
		a[i]=i;
	}
	gawk_srand();
	i=size-1;
	while(i>=0){
		r = (gawk_rand()*i);
		//printf("%d\n", a[r]);
		deck[a[r]] = *tmp;
		//printf("%d\n", deck[r].r);
		a[r]=a[i];
		tmp=tmp->next;
		i--;
	}
	tmp = deck;
	for(i=0;i<52;i++){
		tmp->prev = tmp-1;
		tmp->next = ++tmp;
	}
	(--tmp)->next=NULL;
	deck->prev=NULL;
	//print_deck(deck);

	//exit(0);
	return(deck);
}

//Check deck contains all 52 cards, return 1 if ok, 0 otherwise
int check_deck(struct card* deck)
{
	struct card* ordered_deck;
	struct card* tmp;
	if(get_length_of_deck(deck)!=52){
		return 0;
	}
	ordered_deck=generate_deck();
	tmp = ordered_deck;
	while(tmp!=NULL){
		if(find_card(tmp, deck)==NULL){
			free(ordered_deck);
			return 0;
		}
		tmp=tmp->next;
	}
	free(ordered_deck);
	return(1);

}

//Find first occurrence of card in a deck, or return NULL if not found
struct card *find_card(struct card* card, struct card* deck)
{
	while(deck!=NULL){
		if(deck->s==card->s && deck->r==card->r){
			return(deck);
		}
		deck=deck->next;
	}
	return(NULL);
}
